
describe 'RedditAdapter Testing', ->

  RedditAdapter = modLoader.require('adapters/reddit')
  SubReddit = modLoader.require('models/sub_reddit')
  Post      = modLoader.require('models/post')
  Comment   = modLoader.require('models/comment')

  logger = modLoader.require('libs/logger').getInstance('RedditAdapter')
  mocks  = modLoader.require('mocks')

  it 'should be able to detect a user is not signed in', ->
    adapter = RedditAdapter.create(mocks.ajax, logger)
    expect(adapter.isLoggedIn()).toBe(false)
    return

  it 'should be able to sign a user in', ->
    ajaxStub = jasmine.createSpy('AJAX Login Success').andCallFake (request) ->
      expect(request.url).toBe('https://ssl.reddit.com/api/login/user')
      expect(request.dataType).toBe('json')
      expect(request.type).toBe('POST')
      expect(request.data.passwd).toBe('password')
      expect(request.data.user).toBe('user')
      request.success({uh:'valid id'})
      return

    signedIn = null
    adapter = RedditAdapter.create(ajaxStub, logger)
    adapter.loginUser('user','password')
        .done(
          (response)->
            signedIn = true
            return
        )

    waitsFor ->
      return signedIn
    ,'log in user', 100

    runs ->
      expect(adapter.isLoggedIn()).toBe(true)
      return

    return

  it 'should be able to get a list of all reddits by type', ->
    ajaxStub = jasmine.createSpy('AJAX List Reddits').andCallFake (request) ->
      expect(request.url).toBe('http://www.reddit.com/reddits/popular/.json')
      expect(request.type).toBe('GET')
      expect(request.data.after).toBe(null)
      request.success(mocks.rsp_reddits)
      return

    rspData = null
    adapter = RedditAdapter.create(ajaxStub, logger)
    adapter.listAllSubReddits(RedditAdapter.PAGINATE_TYPE.AFTER, null, RedditAdapter.REDDITS_SORT_TYPES.POPULAR)
        .done(
          (response)->
            rspData = response
            return
        )

    waitsFor ->
      return rspData
    ,'list sub reddits', 100

    runs ->
      expect(rspData.data.length).toBe(2)
      expect(rspData.data[0].title()).toBe('Pictures and Images')
      expect(rspData.data[1].title()).toBe('funny')
      expect(rspData.before).toBe(null)
      expect(rspData.after).toBe('t5_2qq5c')
      return

    return

  it 'should be able to get a list of a users subscribed reddits', ->
    ajaxStub = jasmine.createSpy('AJAX List Subscribed Reddits').andCallFake (request) ->
      expect(request.url).toBe('http://www.reddit.com/reddits/mine/subscriber')
      expect(request.type).toBe('POST')
      expect(request.data.after).toBe(null)
      expect(request.data.uh).toBe('valid user')
      request.success(mocks.rsp_reddits)
      return

    rspData = null
    adapter = RedditAdapter.create(ajaxStub, logger)
    adapter.userSession('valid user')
    adapter.listUserSubReddits(RedditAdapter.PAGINATE_TYPE.AFTER, null, RedditAdapter.OWN_REDDITS_TYPES.SUB)
        .done(
          (response)->
            rspData = response
            return
        )

    waitsFor ->
      return rspData
    ,'list sub reddits', 100

    runs ->
      expect(rspData.data.length).toBe(2)
      expect(rspData.data[0].title()).toBe('Pictures and Images')
      expect(rspData.data[1].title()).toBe('funny')
      expect(rspData.before).toBe(null)
      expect(rspData.after).toBe('t5_2qq5c')
      return

    return

  it 'should be able to get a list of posts from a subreddit ', ->
    ajaxStub = jasmine.createSpy('AJAX List sub reddit posts').andCallFake (request) ->
      expect(request.url).toBe('http://www.reddit.com/r/programming/hot/.json')
      expect(request.type).toBe('GET')
      expect(request.data.after).toBe(null)
      request.success(mocks.rsp_posts)
      return

    rspData = null
    subReddit = SubReddit.create(mocks.model_subreddit)
    adapter = RedditAdapter.create(ajaxStub, logger)
    adapter.listSubRedditPost(RedditAdapter.PAGINATE_TYPE.AFTER, null, subReddit, RedditAdapter.POST_SORT_TYPES.HOT)
        .done(
          (response)->
            rspData = response
            return
        )

    waitsFor ->
      return rspData
    ,'list sub reddits', 100

    runs ->
      expect(rspData.data.length).toBe(2)
      expect(rspData.data[0].title()).toBe('Finding a lost dog\'s owner with Perl')
      expect(rspData.data[1].title()).toBe('Eleven equations every Computer Science geeks should know')
      expect(rspData.before).toBe(null)
      expect(rspData.after).toBe('t3_mqs0s')
      return

    return


  return