require 'fileutils'

namespace :devo do
  task :init_build_dirs do
    dirs = []
    dirs << BUILD_PATH+'/javascripts'
    dirs << BUILD_PATH+'/stylesheets'
    dirs << BUILD_PATH+'/images'
    dirs << BUILD_PATH+'/partials'

    dirs.each do |path|
      begin
        FileUtils::mkdir_p(path) if (!FileTest::directory?(path))
      rescue
        puts "Failed to create directory #{path}"
      end
    end
  end

  task :spec_coffee do
    `coffee -o #{SPEC_PATH}/javascripts -c #{SRC_SPEC_PATH}/coffee`
  end

  task :coffee do
    `coffee -o #{BUILD_PATH}/javascripts -c #{SRC_PATH}/coffee`
  end

  task :sass do
    `sass --update #{SRC_PATH}/sass:#{BUILD_PATH}/stylesheets`
  end

  task :images do
    begin
      FileUtils::cp_r SRC_PATH+'/images', BUILD_PATH
    rescue
      puts "Failed to copy directory #{SRC_PATH}/images to #{BUILD_PATH}"
    end
  end

  task :views do
    begin
      FileUtils::cp_r SRC_PATH+'/views/.', BUILD_PATH
    rescue
      puts "Failed to copy directory #{SRC_PATH}/views to #{BUILD_PATH}"
    end
  end

  task :vendor do
    Dir.glob(VENDOR_PATH+'/*/').map{ |a| File.basename(a) }.each do |dir|
      begin
        FileUtils::rm_rf BUILD_PATH+'/'+dir+'/vendor'
        FileUtils::cp_r VENDOR_PATH+'/'+dir, BUILD_PATH+'/'+dir+'/vendor'
      rescue
        puts "Failed to copy directory #{VENDOR_PATH}/#{dir} to #{BUILD_PATH}/#{dir}/vendor"
      end
    end
  end

  desc 'Cleans the build path'
  task :clean do
    begin
      FileUtils::rm_rf BUILD_PATH
      FileUtils::rm_rf Dir.glob(SPEC_PATH+'/javascripts/*.js')
    rescue
      puts "Failed to clean #{BUILD_PATH}"
    end
  end

  desc 'Compiles all source and writes the output to the build path'
  task :compile => [:init_build_dirs,:coffee,:spec_coffee,:sass,:images,:views,:vendor] do
  end

  desc 'Builds the existing source'
  task :build => [:compile, :package] do
  end

  desc 'Cleans the existing build and rebuilds it'
  task :rebuild => [:clean, :compile,] do
  end

  desc 'Packages the application into a distributable file'
  task :package => [:rebuild, 'jasmine:ci'] do
  end

  desc 'Runs the guard command within bundler'
  task :guard => [:rebuild] do
    `bundle exec guard`
  end

  desc 'Starts the application locally'
  task :app => [:rebuild] do
    `ruby #{ROOT}/webapp/app.rb`
  end
end